@@ -10,6 +10,7 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.INonPerpetualConstruction
 import com.unciv.models.ruleset.Victory
+import com.unciv.models.ruleset.nation.PersonalityValue
 import com.unciv.models.ruleset.tile.ResourceType
 import com.unciv.models.ruleset.tile.TileImprovement
 import com.unciv.models.ruleset.unique.LocalUniqueCache
@@ -22,14 +23,14 @@ import com.unciv.ui.screens.victoryscreen.RankingType
 
 object Automation {
 
-    fun rankTileForCityWork(tile: Tile, city: City, cityStats: Stats, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Float {
+    fun rankTileForCityWork(tile: Tile, city: City, localUniqueCache: LocalUniqueCache = LocalUniqueCache(false)): Float {
         val stats = tile.stats.getTileStats(city, city.civ, localUniqueCache)
-        return rankStatsForCityWork(stats, city, cityStats, false, localUniqueCache)
+        return rankStatsForCityWork(stats, city, false, localUniqueCache)
     }
 
-    fun rankSpecialist(specialist: String, city: City, cityStats: Stats, localUniqueCache: LocalUniqueCache): Float {
+    fun rankSpecialist(specialist: String, city: City, localUniqueCache: LocalUniqueCache): Float {
         val stats = city.cityStats.getStatsOfSpecialist(specialist, localUniqueCache)
-        var rank = rankStatsForCityWork(stats, city, cityStats, true, localUniqueCache)
+        var rank = rankStatsForCityWork(stats, city, true, localUniqueCache)
         // derive GPP score
         var gpp = 0f
         if (city.getRuleset().specialists.containsKey(specialist)) { // To solve problems in total remake mods
@@ -42,11 +43,13 @@ object Automation {
     }
 
 
-    private fun rankStatsForCityWork(stats: Stats, city: City, cityStats: Stats, specialist: Boolean, localUniqueCache: LocalUniqueCache): Float {
+    fun rankStatsForCityWork(stats: Stats, city: City, areWeRankingSpecialist: Boolean, localUniqueCache: LocalUniqueCache): Float {
         val cityAIFocus = city.getCityFocus()
         val yieldStats = stats.clone()
+        val civPersonality = city.civ.getPersonality()
+        val cityStatsObj = city.cityStats
 
-        if (specialist) {
+        if (areWeRankingSpecialist) {
             // If you have the Food Bonus, count as 1 extra food production (base is 2food)
             for (unique in localUniqueCache.forCityGetMatchingUniques(city, UniqueType.FoodConsumptionBySpecialists))
                 if (city.matchesFilter(unique.params[1]))
@@ -58,7 +61,13 @@ object Automation {
             if (city.civ.getHappiness() < 0) yieldStats.happiness *= 2  // double weight for unhappy civilization
         }
 
-        val surplusFood = cityStats[Stat.Food]
+        val surplusFood = city.cityStats.currentCityStats[Stat.Food]
+        // If current Production converts Food into Production, then calculate increased Production Yield
+        if (cityStatsObj.canConvertFoodToProduction(surplusFood, city.cityConstructions.getCurrentConstruction())) {
+            // calculate delta increase of food->prod. This isn't linear
+            yieldStats.production += cityStatsObj.getProductionFromExcessiveFood(surplusFood+yieldStats.food) - cityStatsObj.getProductionFromExcessiveFood(surplusFood)
+            yieldStats.food = 0f  // all food goes to 0
+        }
         // Apply base weights
         yieldStats.applyRankingWeights()
 
@@ -85,15 +94,23 @@ object Automation {
         } else {
             if (city.civ.gold < 0 && city.civ.stats.statsForNextTurn.gold <= 0)
                 yieldStats.gold *= 2 // We have a global problem
+            yieldStats.gold *= civPersonality.scaledFocus(PersonalityValue.Gold)
 
             if (city.tiles.size < 12 || city.civ.wantsToFocusOn(Victory.Focus.Culture))
                 yieldStats.culture *= 2
+            yieldStats.culture *= civPersonality.scaledFocus(PersonalityValue.Culture)
 
-            if (city.civ.getHappiness() < 0 && !specialist) // since this doesn't get updated, may overshoot
+            if (city.civ.getHappiness() < 0 && !areWeRankingSpecialist) // since this doesn't get updated, may overshoot
                 yieldStats.happiness *= 2
+            yieldStats.happiness *= civPersonality.scaledFocus(PersonalityValue.Happiness)
 
             if (city.civ.wantsToFocusOn(Victory.Focus.Science))
                 yieldStats.science *= 2
+            yieldStats.science *= civPersonality.scaledFocus(PersonalityValue.Science)
+
+            yieldStats.production *= civPersonality.scaledFocus(PersonalityValue.Production)
+            yieldStats.faith *= civPersonality.scaledFocus(PersonalityValue.Faith)
+            yieldStats.food *= civPersonality.scaledFocus(PersonalityValue.Food)
         }
 
         // Apply City focus
@@ -359,7 +376,7 @@ object Automation {
         return city.getTiles().filter {
             it.improvementFunctions.canBuildImprovement(improvement, city.civ)
         }.maxByOrNull {
-            rankTileForCityWork(it, city, city.cityStats.currentCityStats, localUniqueCache)
+            rankTileForCityWork(it, city, localUniqueCache)
         }
     }
 
@@ -451,6 +468,7 @@ object Automation {
         rank += stats.production
         rank += stats.science
         rank += stats.culture
+        rank += stats.faith
         return rank
     }
 }
