@@ -5,18 +5,20 @@ import com.unciv.logic.battle.Battle
 import com.unciv.logic.battle.BattleDamage
 import com.unciv.logic.battle.CityCombatant
 import com.unciv.logic.battle.MapUnitCombatant
-import com.unciv.logic.city.City
 import com.unciv.logic.battle.TargetHelper
+import com.unciv.logic.city.City
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.models.ruleset.unique.UniqueType
 
 object BattleHelper {
 
+    /** Returns true if the unit cannot further move this turn - NOT if an attack was successful! */
     fun tryAttackNearbyEnemy(unit: MapUnit, stayOnTile: Boolean = false): Boolean {
         if (unit.hasUnique(UniqueType.CannotAttack)) return false
+        val distanceToTiles = unit.movement.getDistanceToTiles()
         val attackableEnemies = TargetHelper.getAttackableEnemies(unit, unit.movement.getDistanceToTiles(), stayOnTile=stayOnTile)
-            // Only take enemies we can fight without dying
-            .filter {
+            // Only take enemies we can fight without dying or are made to die
+            .filter {unit.hasUnique(UniqueType.SelfDestructs) ||
                 BattleDamage.calculateDamageToAttacker(
                     MapUnitCombatant(unit),
                     Battle.getMapCombatantOfTile(it.tileToAttack)!!
@@ -26,7 +28,14 @@ object BattleHelper {
         val enemyTileToAttack = chooseAttackTarget(unit, attackableEnemies)
 
         if (enemyTileToAttack != null) {
-            Battle.moveAndAttack(MapUnitCombatant(unit), enemyTileToAttack)
+            if (enemyTileToAttack.tileToAttack.militaryUnit == null && unit.baseUnit.isRanged()
+                && unit.movement.canMoveTo(enemyTileToAttack.tileToAttack)
+                && distanceToTiles.containsKey(enemyTileToAttack.tileToAttack)) { // Since the 'getAttackableEnemies' could return a tile we attack at range but cannot reach
+                // Ranged units should move to caputre a civilian unit instead of attacking it
+                unit.movement.moveToTile(enemyTileToAttack.tileToAttack)
+            } else {
+                Battle.moveAndAttack(MapUnitCombatant(unit), enemyTileToAttack)
+            }
         }
         return unit.currentMovement == 0f
     }
@@ -63,7 +72,7 @@ object BattleHelper {
         var attackTile: AttackableTile? = null
         // We always have to calculate the attack value even if there is only one attackableEnemy
         for (attackableEnemy in attackableEnemies) {
-            val tempAttackValue = if (attackableEnemy.tileToAttack.isCityCenter()) 
+            val tempAttackValue = if (attackableEnemy.tileToAttack.isCityCenter())
                 getCityAttackValue(unit, attackableEnemy.tileToAttack.getCity()!!)
             else getUnitAttackValue(unit, attackableEnemy)
             if (tempAttackValue > highestAttackValue) {
@@ -84,15 +93,15 @@ object BattleHelper {
     private fun getCityAttackValue(attacker: MapUnit, city: City): Int {
         val attackerUnit = MapUnitCombatant(attacker)
         val cityUnit = CityCombatant(city)
-        val isCityCapturable = city.health == 1 
+        val isCityCapturable = city.health == 1
             || attacker.baseUnit.isMelee() && city.health <= BattleDamage.calculateDamageToDefender(attackerUnit, cityUnit).coerceAtLeast(1)
         if (isCityCapturable)
             return if (attacker.baseUnit.isMelee()) 10000 // Capture the city immediatly!
             else 0 // Don't attack the city anymore since we are a ranged unit
-        
+
         if (attacker.baseUnit.isMelee()) {
             val battleDamage = BattleDamage.calculateDamageToAttacker(attackerUnit, cityUnit)
-            if (attacker.health - battleDamage * 2 <= 0) {
+            if (attacker.health - battleDamage * 2 <= 0 && !attacker.hasUnique(UniqueType.SelfDestructs)) {
                 // The more fiendly units around the city, the more willing we should be to just attack the city
                 val friendlyUnitsAroundCity = city.getCenterTile().getTilesInDistance(3).count { it.militaryUnit?.civ == attacker.civ }
                 // If we have more than 4 other units around the city, go for it
@@ -104,7 +113,7 @@ object BattleHelper {
             }
         }
 
-        var attackValue = 100 
+        var attackValue = 100
         // Siege units should really only attack the city
         if (attacker.baseUnit.isProbablySiegeUnit()) attackValue += 100
         // Ranged units don't take damage from the city
@@ -122,7 +131,7 @@ object BattleHelper {
                     attackValue += 15
             }
         }
-        
+
         return attackValue
     }
 
@@ -139,7 +148,7 @@ object BattleHelper {
         if (militaryUnit != null) {
             attackValue = 100
             // Associate enemy units with number of hits from this unit to kill them
-            val attacksToKill = (militaryUnit.health.toFloat() / 
+            val attacksToKill = (militaryUnit.health.toFloat() /
                 BattleDamage.calculateDamageToDefender(MapUnitCombatant(attacker), MapUnitCombatant(militaryUnit)))
                 .coerceAtLeast(1f).coerceAtMost(10f)
             // We can kill them in this turn
@@ -148,19 +157,21 @@ object BattleHelper {
             else attackValue -= (attacksToKill * 5).toInt()
         } else if (civilianUnit != null) {
             attackValue = 50
-            // Only melee units should really attack/capture civilian units, ranged units take more than one turn
-            if (attacker.baseUnit.isMelee()) {
+            // Only melee units should really attack/capture civilian units, ranged units may be able to capture by moving
+            if (attacker.baseUnit.isMelee() || attacker.movement.canReachInCurrentTurn(attackTile.tileToAttack)) {
                 if (civilianUnit.isGreatPerson()) {
                     attackValue += 150
                 }
                 if (civilianUnit.hasUnique(UniqueType.FoundCity)) attackValue += 60
+            } else if (attacker.baseUnit.isRanged() && !civilianUnit.hasUnique(UniqueType.Uncapturable)) {
+                return 10 // Don't shoot civilians that we can capture!
             }
         }
         // Prioritise closer units as they are generally more threatening to this unit
         // Moving around less means we are straying less into enemy territory
         // Average should be around 2.5-5 early game and up to 35 for tanks in late game
         attackValue += (attackTile.movementLeftAfterMovingToAttackTile * 5).toInt()
-        
+
         return attackValue
     }
 }
