@@ -10,7 +10,8 @@ import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 
 object CivilianUnitAutomation {
 
-    fun shouldClearTileForAddInCapitalUnits(unit: MapUnit, tile:Tile) = tile.getCity()?.isCapital() == true
+    fun shouldClearTileForAddInCapitalUnits(unit: MapUnit, tile: Tile) =
+        tile.getCity()?.isCapital() == true
         && !unit.hasUnique(UniqueType.AddInCapital)
         && unit.civ.units.getCivUnits().any { unit.hasUnique(UniqueType.AddInCapital) }
 
@@ -28,8 +29,8 @@ object CivilianUnitAutomation {
         if (unit.hasUnique(UniqueType.FoundCity))
             return SpecificUnitAutomation.automateSettlerActions(unit, dangerousTiles)
 
-        if(unit.isAutomatingRoadConnection())
-            return unit.civ.getWorkerAutomation().automateConnectRoad(unit, dangerousTiles)
+        if (unit.isAutomatingRoadConnection())
+            return unit.civ.getWorkerAutomation().roadToAutomation.automateConnectRoad(unit, dangerousTiles)
 
         if (unit.cache.hasUniqueToBuildImprovements)
             return unit.civ.getWorkerAutomation().automateWorkerAction(unit, dangerousTiles)
