@@ -5,7 +5,6 @@ import com.unciv.models.Counter
 import com.unciv.models.ruleset.Building
 import com.unciv.models.ruleset.IConstruction
 import com.unciv.models.ruleset.INonPerpetualConstruction
-import com.unciv.models.ruleset.ModOptionsConstants
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
@@ -205,7 +204,7 @@ class CityStats(val city: City) {
 
         val cityStateStatsMultipliers = city.civ.getMatchingUniques(UniqueType.BonusStatsFromCityStates).toList()
 
-        fun addUniqueStats(unique:Unique) {
+        fun addUniqueStats(unique: Unique) {
             val stats = unique.stats.clone()
             if (unique.sourceObjectType==UniqueTarget.CityState)
                 for (multiplierUnique in cityStateStatsMultipliers)
@@ -245,7 +244,7 @@ class CityStats(val city: City) {
     private fun getStatsPercentBonusesFromUniquesBySource(currentConstruction: IConstruction): StatTreeNode {
         val sourceToStats = StatTreeNode()
 
-        fun addUniqueStats(unique:Unique, stat:Stat, amount:Float) {
+        fun addUniqueStats(unique: Unique, stat: Stat, amount: Float) {
             sourceToStats.addStats(Stats().add(stat, amount), unique.getSourceNameForUser(), unique.sourceObjectName ?: "")
         }
 
@@ -340,7 +339,7 @@ class CityStats(val city: City) {
     //endregion
     //region State-Changing Methods
 
-    fun updateTileStats(localUniqueCache:LocalUniqueCache = LocalUniqueCache()) {
+    fun updateTileStats(localUniqueCache: LocalUniqueCache = LocalUniqueCache()) {
         val stats = Stats()
         val workedTiles = city.tilesInRange.asSequence()
             .filter {
@@ -523,7 +522,7 @@ class CityStats(val city: City) {
         }
 
         // AFTER we've gotten all the gold stats figured out, only THEN do we plonk that gold into Science
-        if (city.getRuleset().modOptions.uniques.contains(ModOptionsConstants.convertGoldToScience)) {
+        if (city.getRuleset().modOptions.hasUnique(UniqueType.ConvertGoldToScience)) {
             val amountConverted = (newFinalStatList.values.sumOf { it.gold.toDouble() }
                     * city.civ.tech.goldPercentConvertedToScience).toInt().toFloat()
             if (amountConverted > 0) // Don't want you converting negative gold to negative science yaknow
@@ -578,10 +577,7 @@ class CityStats(val city: City) {
         val buildingsMaintenance = getBuildingMaintenanceCosts() // this is AFTER the bonus calculation!
         newFinalStatList["Maintenance"] = Stats(gold = -buildingsMaintenance.toInt().toFloat())
 
-        if (totalFood > 0
-            && currentConstruction is INonPerpetualConstruction
-            && currentConstruction.hasUnique(UniqueType.ConvertFoodToProductionWhenConstructed)
-        ) {
+        if (canConvertFoodToProduction(totalFood, currentConstruction)) {
             newFinalStatList["Excess food to production"] =
                 Stats(production = getProductionFromExcessiveFood(totalFood), food = -totalFood)
         }
@@ -604,9 +600,20 @@ class CityStats(val city: City) {
         finalStatList = newFinalStatList
     }
 
-    // calculate the conversion of the excessive food to the production
-    // See for details: https://civilization.fandom.com/wiki/Settler_(Civ5)
-    private fun getProductionFromExcessiveFood(food : Float): Float {
+    fun canConvertFoodToProduction(food: Float, currentConstruction: IConstruction): Boolean {
+        return (food > 0
+            && currentConstruction is INonPerpetualConstruction
+            && currentConstruction.hasUnique(UniqueType.ConvertFoodToProductionWhenConstructed))
+    }
+
+    /**
+     * Calculate the conversion of the excessive food to production when
+     * [UniqueType.ConvertFoodToProductionWhenConstructed] is at front of the build queue
+     * @param food is amount of excess Food generates this turn
+     * See for details: https://civilization.fandom.com/wiki/Settler_(Civ5)
+     * @see calcFoodEaten as well for Food consumed this turn
+     */
+    fun getProductionFromExcessiveFood(food : Float): Float {
         return if (food >= 4.0f ) 2.0f + (food / 4.0f).toInt()
           else if (food >= 2.0f ) 2.0f
           else if (food >= 1.0f ) 1.0f
