@@ -31,18 +31,8 @@ class RuinsManager(
     }
 
     private fun getShuffledPossibleRewards(triggeringUnit: MapUnit): Iterable<RuinReward> {
-        val stateForOnlyAvailableWhen = StateForConditionals(civInfo, unit = triggeringUnit, tile = triggeringUnit.getTile())
         val candidates =
-            validRewards.asSequence()
-            // Filter out what shouldn't be considered right now, before the random choice
-            .filterNot { possibleReward ->
-                possibleReward.name in lastChosenRewards
-                    || civInfo.gameInfo.difficulty in possibleReward.excludedDifficulties
-                    || possibleReward.hasUnique(UniqueType.HiddenWithoutReligion) && !civInfo.gameInfo.isReligionEnabled()
-                    || possibleReward.hasUnique(UniqueType.HiddenAfterGreatProphet) && civInfo.religionManager.greatProphetsEarned() > 0
-                    || possibleReward.getMatchingUniques(UniqueType.OnlyAvailableWhen, StateForConditionals.IgnoreConditionals)
-                    .any { !it.conditionalsApply(stateForOnlyAvailableWhen) }
-            }
+            validRewards.asSequence().filter { isPossibleReward(it, triggeringUnit) }
             // This might be a dirty way to do this, but it works (we do have randomWeighted in CollectionExtensions, but below we
             // need to choose another when the first choice's TriggerActivations report failure, and that's simpler this way)
             // For each possible reward, this feeds (reward.weight) copies of this reward to the overall Sequence to implement 'weight'.
@@ -55,14 +45,25 @@ class RuinsManager(
         return candidates
     }
 
+    private fun isPossibleReward(ruinReward: RuinReward, unit: MapUnit): Boolean {
+        if (ruinReward.name in lastChosenRewards) return false
+        if (civInfo.gameInfo.difficulty in ruinReward.excludedDifficulties) return false
+        val stateForConditionals = StateForConditionals(civInfo, unit = unit, tile = unit.getTile())
+        if (ruinReward.hasUnique(UniqueType.HiddenWithoutReligion, stateForConditionals) && !civInfo.gameInfo.isReligionEnabled()) return false
+        if (ruinReward.hasUnique(UniqueType.HiddenAfterGreatProphet, stateForConditionals) && civInfo.religionManager.greatProphetsEarned() > 0) return false
+        if (ruinReward.hasUnique(UniqueType.Unavailable, stateForConditionals)) return false
+        if (ruinReward.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
+                .any { !it.conditionalsApply(stateForConditionals) }) return false
+        return true
+    }
+
     fun selectNextRuinsReward(triggeringUnit: MapUnit) {
         for (possibleReward in getShuffledPossibleRewards(triggeringUnit)) {
             var atLeastOneUniqueHadEffect = false
             for (unique in possibleReward.uniqueObjects) {
                 atLeastOneUniqueHadEffect =
                     atLeastOneUniqueHadEffect
-                    || UniqueTriggerActivation.triggerCivwideUnique(unique, civInfo, tile = triggeringUnit.getTile(), notification = possibleReward.notification, triggerNotificationText = "from the ruins")
-                    || UniqueTriggerActivation.triggerUnitwideUnique(unique, triggeringUnit, notification = possibleReward.notification)
+                    || UniqueTriggerActivation.triggerUnique(unique, triggeringUnit, notification = possibleReward.notification)
             }
             if (atLeastOneUniqueHadEffect) {
                 rememberReward(possibleReward.name)
