@@ -151,18 +151,10 @@ class NaturalWonderGenerator(val ruleset: Ruleset, val randomness: MapGeneration
         val targetGroupSize = if (minGroupSize == maxGroupSize) maxGroupSize
             else (minGroupSize..maxGroupSize).random(randomness.RNG)
 
-        var convertNeighborsExcept: String? = null
-        var convertUnique = wonder.getMatchingUniques(UniqueType.NaturalWonderConvertNeighbors).firstOrNull()
-        var convertNeighborsTo = convertUnique?.params?.get(0)
-        if (convertNeighborsTo == null) {
-            convertUnique = wonder.getMatchingUniques(UniqueType.NaturalWonderConvertNeighborsExcept).firstOrNull()
-            convertNeighborsExcept = convertUnique?.params?.get(0)
-            convertNeighborsTo = convertUnique?.params?.get(1)
-        }
-
         if (suitableLocations.size >= minGroupSize) {
             val location = suitableLocations.random(randomness.RNG)
             val list = mutableListOf(location)
+
             while (list.size < targetGroupSize) {
                 val allNeighbors = list.flatMap { it.neighbors }.minus(list).toHashSet()
                 val candidates = suitableLocations.filter { it in allNeighbors }
@@ -170,31 +162,12 @@ class NaturalWonderGenerator(val ruleset: Ruleset, val randomness: MapGeneration
                 list.add(candidates.random(randomness.RNG))
             }
             if (list.size >= minGroupSize) {
-                list.forEach {
-                    clearTile(it)
-                    it.naturalWonder = wonder.name
-                    if (wonder.turnsInto != null)
-                        it.baseTerrain = wonder.turnsInto!!
+                for (tileToConvert in list) {
+                    placeNaturalWonder(wonder, tileToConvert)
                     // Add all tiles within a certain distance to a blacklist so NW:s don't cluster
-                    blockedTiles.addAll(it.getTilesInDistance(it.tileMap.mapParameters.mapSize.height / 5))
-                }
-                if (convertNeighborsTo != null) {
-                    for (tile in location.neighbors) {
-                        if (tile.baseTerrain == convertNeighborsTo) continue
-                        if (tile.baseTerrain == convertNeighborsExcept) continue
-                        if (convertNeighborsTo == Constants.coast)
-                            for (neighbor in tile.neighbors) {
-                                // This is so we don't have this tile turn into Coast, and then it's touching a Lake tile.
-                                // We just turn the lake tiles into this kind of tile.
-                                if (neighbor.baseTerrain == Constants.lakes) {
-                                    neighbor.baseTerrain = tile.baseTerrain
-                                    neighbor.setTerrainTransients()
-                                }
-                            }
-                        tile.baseTerrain = convertNeighborsTo
-                        clearTile(tile)
-                    }
+                    blockedTiles.addAll(tileToConvert.getTilesInDistance(tileToConvert.tileMap.mapParameters.mapSize.height / 5))
                 }
+
                 debug("Natural Wonder %s @%s", wonder.name, location.position)
 
                 return true
@@ -205,11 +178,47 @@ class NaturalWonderGenerator(val ruleset: Ruleset, val randomness: MapGeneration
         return false
     }
 
-    private fun clearTile(tile: Tile) {
-        tile.setTerrainFeatures(listOf())
-        tile.resource = null
-        tile.removeImprovement()
-        tile.setTerrainTransients()
+    companion object {
+        fun placeNaturalWonder(wonder: Terrain, location: Tile) {
+            clearTile(location)
+            location.naturalWonder = wonder.name
+            if (wonder.turnsInto != null)
+                location.baseTerrain = wonder.turnsInto!!
+
+            var convertNeighborsExcept: String? = null
+            var convertUnique = wonder.getMatchingUniques(UniqueType.NaturalWonderConvertNeighbors).firstOrNull()
+            var convertNeighborsTo = convertUnique?.params?.get(0)
+            if (convertNeighborsTo == null) {
+                convertUnique = wonder.getMatchingUniques(UniqueType.NaturalWonderConvertNeighborsExcept).firstOrNull()
+                convertNeighborsExcept = convertUnique?.params?.get(0)
+                convertNeighborsTo = convertUnique?.params?.get(1)
+            }
+
+            if (convertNeighborsTo != null) {
+                for (tile in location.neighbors) {
+                    if (tile.baseTerrain == convertNeighborsTo) continue
+                    if (tile.baseTerrain == convertNeighborsExcept) continue
+                    if (convertNeighborsTo == Constants.coast)
+                        for (neighbor in tile.neighbors) {
+                            // This is so we don't have this tile turn into Coast, and then it's touching a Lake tile.
+                            // We just turn the lake tiles into this kind of tile.
+                            if (neighbor.baseTerrain == Constants.lakes) {
+                                neighbor.baseTerrain = tile.baseTerrain
+                                neighbor.setTerrainTransients()
+                            }
+                        }
+                    tile.baseTerrain = convertNeighborsTo
+                    clearTile(tile)
+                }
+            }
+        }
+
+        private fun clearTile(tile: Tile) {
+            tile.setTerrainFeatures(listOf())
+            tile.resource = null
+            tile.removeImprovement()
+            tile.setTerrainTransients()
+        }
     }
 
     /** Implements [UniqueParameterType.SimpleTerrain][com.unciv.models.ruleset.unique.UniqueParameterType.SimpleTerrain] */
