@@ -53,6 +53,9 @@ class TileInfoImprovementFunctions(val tile: Tile) {
         else if (improvement.hasUnique(UniqueType.Unbuildable, stateForConditionals))
             yield(ImprovementBuildingProblem.ConditionallyUnbuildable)
 
+        if (improvement.hasUnique(UniqueType.Unavailable, stateForConditionals))
+            yield(ImprovementBuildingProblem.ConditionallyUnbuildable)
+
         if (tile.getOwner() != civInfo && !improvement.hasUnique(UniqueType.CanBuildOutsideBorders, stateForConditionals)) {
             if (!improvement.hasUnique(UniqueType.CanBuildJustOutsideBorders, stateForConditionals))
                 yield(ImprovementBuildingProblem.OutsideBorders)
@@ -60,7 +63,7 @@ class TileInfoImprovementFunctions(val tile: Tile) {
                 yield(ImprovementBuildingProblem.NotJustOutsideBorders)
         }
 
-        if (improvement.getMatchingUniques(UniqueType.OnlyAvailableWhen, StateForConditionals.IgnoreConditionals)
+        if (improvement.getMatchingUniques(UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals)
                     .any { !it.conditionalsApply(stateForConditionals) })
             yield(ImprovementBuildingProblem.UnmetConditional)
 
@@ -76,10 +79,9 @@ class TileInfoImprovementFunctions(val tile: Tile) {
                     .any { civInfo.getResourceAmount(it.params[1]) < it.params[0].toInt() })
             yield(ImprovementBuildingProblem.MissingResources)
 
-        val knownFeatureRemovals = tile.ruleset.tileImprovements.values
+        val knownFeatureRemovals = tile.ruleset.tileRemovals
             .filter { rulesetImprovement ->
-                rulesetImprovement.name.startsWith(Constants.remove)
-                        && RoadStatus.values().none { it.removeAction == rulesetImprovement.name }
+                        RoadStatus.values().none { it.removeAction == rulesetImprovement.name }
                         && (rulesetImprovement.techRequired == null || civInfo.tech.isResearched(rulesetImprovement.techRequired!!))
             }
 
@@ -104,16 +106,18 @@ class TileInfoImprovementFunctions(val tile: Tile) {
         ): Boolean {
             val topTerrain = tile.lastTerrain
             // We can build if we are specifically allowed to build on this terrain
-            if (isAllowedOnFeature(topTerrain.name)) return true
+            if (isAllowedOnFeature(topTerrain)) return true
 
             // Otherwise, we can if this improvement removes the top terrain
             if (!hasUnique(UniqueType.RemovesFeaturesIfBuilt, stateForConditionals)) return false
-            val removeAction = tile.ruleset.tileImprovements[Constants.remove + topTerrain.name] ?: return false
-            // and we have the tech to remove that top terrain
-            if (removeAction.techRequired != null && (knownFeatureRemovals == null || removeAction !in knownFeatureRemovals)) return false
-            // and we can build it on the tile without the top terrain
+            if (knownFeatureRemovals.isNullOrEmpty()) return false
+            val featureRemovals = tile.terrainFeatures.mapNotNull { feature -> 
+                tile.ruleset.tileRemovals.firstOrNull { it.name == Constants.remove + feature } }
+            if (featureRemovals.isEmpty()) return false
+            if (featureRemovals.any { it !in knownFeatureRemovals }) return false
             val clonedTile = tile.clone()
-            clonedTile.removeTerrainFeature(topTerrain.name)
+            clonedTile.setTerrainFeatures(tile.terrainFeatures.filterNot {
+                feature -> featureRemovals.any{ it.name.removePrefix(Constants.remove) == feature } })
             return clonedTile.improvementFunctions.canImprovementBeBuiltHere(improvement, resourceIsVisible, knownFeatureRemovals, stateForConditionals)
         }
 
@@ -128,7 +132,7 @@ class TileInfoImprovementFunctions(val tile: Tile) {
             // Can only remove roads if that road is actually there
             RoadStatus.values().any { it.removeAction == improvement.name } -> tile.roadStatus.removeAction == improvement.name
             // Can only remove features or improvement if that feature/improvement is actually there
-            improvement.name.startsWith(Constants.remove) -> tile.terrainFeatures.any { Constants.remove + it == improvement.name } 
+            improvement.name.startsWith(Constants.remove) -> tile.terrainFeatures.any { Constants.remove + it == improvement.name }
                 || Constants.remove + tile.improvement == improvement.name
             // Can only build roads if on land and they are better than the current road
             RoadStatus.values().any { it.name == improvement.name } -> !tile.isWater
@@ -172,7 +176,7 @@ class TileInfoImprovementFunctions(val tile: Tile) {
             // At this point we know this is a normal improvement and that there is no reason not to allow it to be built.
 
             // Lastly we check if the improvement may be built on this terrain or resource
-            improvement.canBeBuiltOn(tile.lastTerrain.name) -> true
+            improvement.isAllowedOnFeature(tile.lastTerrain) -> true
             tile.isLand && improvement.canBeBuiltOn("Land") -> true
             tile.isWater && improvement.canBeBuiltOn("Water") -> true
             // DO NOT reverse this &&. isAdjacentToFreshwater() is a lazy which calls a function, and reversing it breaks the tests.
@@ -210,14 +214,14 @@ class TileInfoImprovementFunctions(val tile: Tile) {
         if (improvementObject != null && improvementObject.hasUnique(UniqueType.RemovesFeaturesIfBuilt)) {
             // Remove terrainFeatures that a Worker can remove
             // and that aren't explicitly allowed under the improvement
-            val removableTerrainFeatures = tile.terrainFeatures.filter { feature ->
-                val removingAction = "${Constants.remove}$feature"
+            val removableTerrainFeatures = tile.terrainFeatureObjects.filter { feature ->
+                val removingAction = "${Constants.remove}${feature.name}"
 
                 removingAction in tile.ruleset.tileImprovements // is removable
                     && !improvementObject.isAllowedOnFeature(feature) // cannot coexist
             }
 
-            tile.setTerrainFeatures(tile.terrainFeatures.filterNot { it in removableTerrainFeatures })
+            tile.setTerrainFeatures(tile.terrainFeatures.filterNot { feature -> removableTerrainFeatures.any { it.name == feature } })
         }
 
         if (civToActivateBroaderEffects != null && improvementObject != null
@@ -229,12 +233,10 @@ class TileInfoImprovementFunctions(val tile: Tile) {
             triggerImprovementUniques(improvementObject, civToActivateBroaderEffects, unit)
 
         val city = tile.owningCity
-        if (city != null) {
+        if (civToActivateBroaderEffects != null && city != null) {
             city.cityStats.update()
-            if (civToActivateBroaderEffects != null) {
-                city.civ.cache.updateCivResources()
-                city.reassignPopulationDeferred()
-            }
+            city.civ.cache.updateCivResources()
+            city.reassignPopulationDeferred()
         }
     }
 
@@ -243,23 +245,20 @@ class TileInfoImprovementFunctions(val tile: Tile) {
         civ: Civilization,
         unit: MapUnit? = null
     ) {
+        val stateForConditionals = StateForConditionals(civ, unit = unit, tile = tile)
+
         for (unique in improvement.uniqueObjects.filter { !it.hasTriggerConditional()
-            && it.conditionalsApply(StateForConditionals(civ)) })
-            if (unit != null) {
-                UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
-            }
-            else UniqueTriggerActivation.triggerCivwideUnique(unique, civ, tile = tile)
+            && it.conditionalsApply(stateForConditionals) })
+            UniqueTriggerActivation.triggerUnique(unique, civ, unit = unit, tile = tile)
 
-        if (unit != null){
-            for (unique in unit.getTriggeredUniques(UniqueType.TriggerUponBuildingImprovement)
-                .filter { improvement.matchesFilter(it.params[0]) })
-                UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
-            }
-
-        for (unique in civ.getMatchingUniques(
-            UniqueType.TriggerUponBuildingImprovement, StateForConditionals(civInfo = civ, unit = unit))
+        for (unique in civ.getTriggeredUniques(UniqueType.TriggerUponBuildingImprovement, stateForConditionals)
             .filter { improvement.matchesFilter(it.params[0]) })
-            UniqueTriggerActivation.triggerCivwideUnique(unique, civ, tile = tile)
+            UniqueTriggerActivation.triggerUnique(unique, civ, unit = unit, tile = tile)
+
+        if (unit == null) return
+        for (unique in unit.getTriggeredUniques(UniqueType.TriggerUponBuildingImprovement, stateForConditionals)
+            .filter { improvement.matchesFilter(it.params[0]) })
+            UniqueTriggerActivation.triggerUnique(unique, civ, unit = unit, tile = tile)
     }
 
     private fun activateRemovalImprovement(
@@ -292,7 +291,7 @@ class TileInfoImprovementFunctions(val tile: Tile) {
         }
     }
 
-    private fun tryProvideProductionToClosestCity(removedTerrainFeature: String, civ:Civilization) {
+    private fun tryProvideProductionToClosestCity(removedTerrainFeature: String, civ: Civilization) {
         val closestCity = civ.cities.minByOrNull { it.getCenterTile().aerialDistanceTo(tile) }
         @Suppress("FoldInitializerAndIfToElvis")
         if (closestCity == null) return
