@@ -1,6 +1,7 @@
 package com.unciv.models.ruleset
 
 import com.badlogic.gdx.files.FileHandle
+import com.unciv.Constants
 import com.unciv.json.fromJsonFile
 import com.unciv.json.json
 import com.unciv.logic.BackwardCompatibility.updateDeprecations
@@ -8,6 +9,7 @@ import com.unciv.models.metadata.BaseRuleset
 import com.unciv.models.ruleset.nation.CityStateType
 import com.unciv.models.ruleset.nation.Difficulty
 import com.unciv.models.ruleset.nation.Nation
+import com.unciv.models.ruleset.nation.Personality
 import com.unciv.models.ruleset.tech.Era
 import com.unciv.models.ruleset.tech.TechColumn
 import com.unciv.models.ruleset.tech.Technology
@@ -69,11 +71,32 @@ class Ruleset {
     val unitTypes = LinkedHashMap<String, UnitType>()
     var victories = LinkedHashMap<String, Victory>()
     var cityStateTypes = LinkedHashMap<String, CityStateType>()
+    val personalities = LinkedHashMap<String, Personality>()
+    val events = LinkedHashMap<String, Event>()
 
-    val greatGeneralUnits by lazy { 
+    val greatGeneralUnits by lazy {
         units.values.filter { it.hasUnique(UniqueType.GreatPersonFromCombat, StateForConditionals.IgnoreConditionals) }
     }
 
+    val tileRemovals by lazy { tileImprovements.values.filter { it.name.startsWith(Constants.remove) } }
+
+    /** Contains all happiness levels that moving *from* them, to one *below* them, can change uniques that apply */
+    val allHappinessLevelsThatAffectUniques by lazy {
+        sequence {
+            for (rulesetObject in (units.values + terrains.values + buildings.values + technologies.values + eras.values
+                + beliefs.values + unitPromotions.values + tileResources.values + policies.values + tileImprovements.values + globalUniques))
+                for (unique in rulesetObject.uniqueObjects)
+                    for (conditional in unique.conditionals){
+                        if (conditional.type == UniqueType.ConditionalBelowHappiness) yield(conditional.params[0].toInt())
+                        if (conditional.type == UniqueType.ConditionalBetweenHappiness){
+                            yield(conditional.params[0].toInt())
+                            yield(conditional.params[1].toInt() + 1)
+                        }
+                        if (conditional.type == UniqueType.ConditionalHappy) yield(0)
+                    }
+        }.toSet()
+    }
+
     val mods = LinkedHashSet<String>()
     var modOptions = ModOptions()
 
@@ -142,6 +165,8 @@ class Ruleset {
                 units.remove(it)
             }
         units.putAll(ruleset.units)
+        personalities.putAll(ruleset.personalities)
+        events.putAll(ruleset.events)
         modOptions.uniques.addAll(ruleset.modOptions.uniques)
         modOptions.constants.merge(ruleset.modOptions.constants)
 
@@ -175,6 +200,8 @@ class Ruleset {
         unitTypes.clear()
         victories.clear()
         cityStateTypes.clear()
+        personalities.clear()
+        events.clear()
     }
 
     fun allRulesetObjects(): Sequence<IRulesetObject> =
@@ -197,7 +224,8 @@ class Ruleset {
             tileResources.values.asSequence() +
             unitPromotions.values.asSequence() +
             units.values.asSequence() +
-            unitTypes.values.asSequence()
+            unitTypes.values.asSequence() +
+            personalities.values.asSequence()
             // Victories is only INamed
     fun allIHasUniques(): Sequence<IHasUniques> =
             allRulesetObjects() + sequenceOf(modOptions)
@@ -357,6 +385,16 @@ class Ruleset {
             cityStateTypes += createHashmap(json().fromJsonFile(Array<CityStateType>::class.java, cityStateTypesFile))
         }
 
+        val personalitiesFile = folderHandle.child("Personalities.json")
+        if (personalitiesFile.exists()) {
+            personalities += createHashmap(json().fromJsonFile(Array<Personality>::class.java, personalitiesFile))
+        }
+
+        val eventsFile = folderHandle.child("Events.json")
+        if (eventsFile.exists()) {
+            events += createHashmap(json().fromJsonFile(Array<Event>::class.java, eventsFile))
+        }
+
 
 
         // Add objects that might not be present in base ruleset mods, but are required
