@@ -51,6 +51,7 @@ class Nation : RulesetObject() {
     var uniqueText = ""
     var innerColor: List<Int>? = null
     var startBias = ArrayList<String>()
+    var personality: String? = null
 
     var startIntroPart1 = ""
     var startIntroPart2 = ""
@@ -108,10 +109,10 @@ class Nation : RulesetObject() {
     }
 
     override fun getCivilopediaTextLines(ruleset: Ruleset): List<FormattedLine> {
-        if (isCityState) return getCityStateInfo(ruleset)
-
         val textList = ArrayList<FormattedLine>()
 
+        if (isCityState) textList += getCityStateInfo(ruleset)
+
         if (leaderName.isNotEmpty()) {
             textList += FormattedLine(extraImage = "LeaderIcons/$leaderName", imageSize = 200f)
             textList += FormattedLine(getLeaderDisplayName(), centered = true, header = 3)
@@ -164,7 +165,7 @@ class Nation : RulesetObject() {
             textList += FormattedLine("{$header:} ")
             for (unique in bonuses) {
                 textList += FormattedLine(unique, indent = 1)
-                if (unique.isOfType(UniqueType.CityStateUniqueLuxury)) showResources = true
+                if (unique.type == UniqueType.CityStateUniqueLuxury) showResources = true
             }
         }
 
@@ -183,6 +184,7 @@ class Nation : RulesetObject() {
                 }
             }
         }
+        textList += FormattedLine(separator = true)
 
         // personality is not a nation property, it gets assigned to the civ randomly
         return textList
@@ -252,8 +254,10 @@ class Nation : RulesetObject() {
                     )
                 }
             }
-            for (unique in improvement.uniques)
+            for (unique in improvement.uniqueObjects) {
+                if (unique.isHiddenToUsers()) continue
                 yield(FormattedLine(unique, indent = 1))
+            }
         }
     }
 
@@ -263,9 +267,9 @@ class Nation : RulesetObject() {
         return MultiFilter.multiFilter(filter, ::matchesSingleFilter)
     }
 
-    fun matchesSingleFilter(filter: String): Boolean {
+    private fun matchesSingleFilter(filter: String): Boolean {
         return when (filter) {
-            "All" -> true
+            in Constants.all -> true
             name -> true
             "Major" -> isMajorCiv
             // "CityState" to be deprecated, replaced by "City-States"
