@@ -181,7 +181,7 @@ class UnitOverviewTab(
                 { it.currentMovement <= Constants.minimumMovementEpsilon },
                 { abs(it.currentTile.position.x) + abs(it.currentTile.position.y) })
         )) {
-            val baseUnit = unit.baseUnit()
+            val baseUnit = unit.baseUnit
 
             // Unit button column - name, health, fortified, sleeping, embarked are visible here
             val button = IconTextButton(
@@ -258,22 +258,9 @@ class UnitOverviewTab(
             add(promotionsTable)
 
             // Upgrade column
-            val unitAction = UnitActionsUpgrade.getUpgradeActionAnywhere(unit)
-            if (unitAction != null) {
-                val enable = unitAction.action != null && viewingPlayer.isCurrentPlayer() &&
-                    GUI.isAllowedChangeState()
-                val unitToUpgradeTo = (unitAction as UpgradeUnitAction).unitToUpgradeTo
-                val selectKey = getUnitIdentifier(unit, unitToUpgradeTo)
-                val upgradeIcon = ImageGetter.getUnitIcon(unitToUpgradeTo.name,
-                    if (enable) Color.GREEN else Color.GREEN.darken(0.5f))
-                if (enable) upgradeIcon.onClick {
-                    UnitUpgradeMenu(overviewScreen.stage, upgradeIcon, unit, unitAction) {
-                        unitListTable.updateUnitListTable()
-                        select(selectKey)
-                    }
-                }
-                add(upgradeIcon).size(28f)
-            } else add()
+            val upgradeTable = Table()
+            updateUpgradeTable(upgradeTable, unit)
+            add(upgradeTable)
 
             // Numeric health column - there's already a health bar on the button, but...?
             if (unit.health < 100) add(unit.health.toLabel()) else add()
@@ -282,6 +269,28 @@ class UnitOverviewTab(
         return this
     }
 
+    private fun updateUpgradeTable(table: Table, unit: MapUnit){
+        table.clearChildren()
+
+        val unitActions = UnitActionsUpgrade.getUpgradeActionAnywhere(unit)
+        if (unitActions.none()) table.add()
+        for (unitAction in unitActions){
+            val enable = unitAction.action != null && viewingPlayer.isCurrentPlayer() &&
+                GUI.isAllowedChangeState()
+            val unitToUpgradeTo = (unitAction as UpgradeUnitAction).unitToUpgradeTo
+            val selectKey = getUnitIdentifier(unit, unitToUpgradeTo)
+            val upgradeIcon = ImageGetter.getUnitIcon(unitToUpgradeTo.name,
+                if (enable) Color.GREEN else Color.GREEN.darken(0.5f))
+            upgradeIcon.onClick {
+                UnitUpgradeMenu(overviewScreen.stage, upgradeIcon, unit, unitAction, enable) {
+                    unitListTable.updateUnitListTable()
+                    select(selectKey)
+                }
+            }
+            table.add(upgradeIcon).size(28f)
+        }
+    }
+
     private fun updatePromotionsTable(table: Table, unit: MapUnit) {
         table.clearChildren()
 
