@@ -18,6 +18,7 @@ import com.unciv.logic.civilization.PopupAlert
 import com.unciv.logic.civilization.diplomacy.DiplomacyFlags
 import com.unciv.logic.civilization.diplomacy.DiplomaticModifiers
 import com.unciv.logic.civilization.diplomacy.RelationshipLevel
+import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.tr
@@ -40,6 +41,10 @@ import java.util.EnumSet
  * [Popup] communicating events other than trade offers to the player.
  * (e.g. First Contact, Wonder built, Tech researched,...)
  *
+ * **Opens itself at the end of instantiation!**
+ *
+ * (In rare cases, it chooses not to: Mods making a RecapturedCivilian not find the unit as it was illegal and removed after the actual capture)
+ *
  * Called in [WorldScreen].update, which pulls them from viewingCiv.popupAlerts.
  *
  * @param worldScreen The parent screen
@@ -73,6 +78,8 @@ class AlertPopup(
         addCloseButton(text, KeyboardBinding.NextTurnAlternate, null)
 
     init {
+        var skipThisAlert = false
+
         // This makes the buttons fill up available width. See comments in #9559.
         // To implement a middle ground, I would either simply replace growX() with minWidth(240f) or so,
         // or replace the Popup.equalizeLastTwoButtonWidths() function with something intelligent not
@@ -95,9 +102,11 @@ class AlertPopup(
             AlertType.StartIntro -> addStartIntro()
             AlertType.DiplomaticMarriage -> addDiplomaticMarriage()
             AlertType.BulliedProtectedMinor, AlertType.AttackedProtectedMinor -> addBulliedOrAttackedProtectedMinor()
-            AlertType.RecapturedCivilian -> addRecapturedCivilian()
+            AlertType.RecapturedCivilian -> skipThisAlert = addRecapturedCivilian()
             AlertType.GameHasBeenWon -> addGameHasBeenWon()
+            AlertType.Event -> skipThisAlert = !addEvent()
         }
+        if (!skipThisAlert) open()
     }
 
     //region AlertType handlers
@@ -279,14 +288,12 @@ class AlertPopup(
         music.chooseTrack(viewingCiv.civName, MusicMood.Golden, MusicTrackChooserFlags.setSpecific)
     }
 
-    private fun addRecapturedCivilian() {
+    /** @return true to skip opening this Popup, as we're running in the initialization phase before the Popup is open */
+    private fun addRecapturedCivilian(): Boolean {
         val position = Vector2().fromString(popupAlert.value)
         val tile = gameInfo.tileMap[position]
-        val capturedUnit = tile.civilianUnit // This has got to be it
-        if (capturedUnit == null) { // the unit disappeared somehow? maybe a modded action?
-            close()
-            return
-        }
+        val capturedUnit = tile.civilianUnit  // This has got to be it
+            ?: return true // the unit disappeared somehow? maybe a modded action?
         val originalOwner = getCiv(capturedUnit.originalOwner!!)
         val captor = viewingCiv
 
@@ -327,6 +334,7 @@ class AlertPopup(
             // Take it for ourselves
             BattleUnitCapture.captureOrConvertToWorker(capturedUnit, captor)
         }
+        return false
     }
 
     private fun addStartIntro() {
@@ -491,6 +499,24 @@ class AlertPopup(
         }
     }
 
+    /** Returns if event was triggered correctly */
+    private fun addEvent(): Boolean {
+        val event = gameInfo.ruleset.events[popupAlert.value] ?: return false
+
+        val civ = gameInfo.currentPlayerCiv
+        val choices = event.choices.filter { it.matchesConditions(StateForConditionals(civ)) }
+        if (choices.isEmpty()) return false
+
+        addGoodSizedLabel(event.text)
+        for (choice in choices){
+            addSeparator()
+            add(choice.text.toTextButton().onActivation { close(); choice.triggerChoice(civ) }).row()
+            for (triggeredUnique in choice.triggeredUniques)
+                addGoodSizedLabel(triggeredUnique).row()
+        }
+        return true
+    }
+
     //endregion
 
     override fun close() {
