@@ -17,6 +17,7 @@ import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.stats.Stat
 import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.removeConditionals
 import com.unciv.models.translations.tr
@@ -24,12 +25,10 @@ import com.unciv.ui.components.fonts.Fonts
 import com.unciv.ui.popups.ConfirmPopup
 import com.unciv.ui.screens.pickerscreens.ImprovementPickerScreen
 
+@Suppress("UNUSED_PARAMETER") // These methods are used as references in UnitActions.actionTypeToFunctions and need identical signature
 object UnitActionsFromUniques {
 
-    fun getFoundCityActions(unit: MapUnit, tile: Tile): List<UnitAction> {
-        val getFoundCityAction = getFoundCityAction(unit, tile) ?: return emptyList()
-        return listOf(getFoundCityAction)
-    }
+    internal fun getFoundCityActions(unit: MapUnit, tile: Tile) = sequenceOf(getFoundCityAction(unit, tile)).filterNotNull()
 
     /** Produce a [UnitAction] for founding a city.
      * @param unit The unit to do the founding.
@@ -39,7 +38,7 @@ object UnitActionsFromUniques {
      * The [action][UnitAction.action] field will be null if the action cannot be done here and now
      * (no movement left, too close to another city).
      */
-    fun getFoundCityAction(unit: MapUnit, tile: Tile): UnitAction? {
+    internal fun getFoundCityAction(unit: MapUnit, tile: Tile): UnitAction? {
         val unique = UnitActionModifiers.getUsableUnitActionUniques(unit, UniqueType.FoundCity)
             .firstOrNull() ?: return null
 
@@ -48,7 +47,7 @@ object UnitActionsFromUniques {
         if (unit.civ.isOneCityChallenger() && unit.civ.hasEverOwnedOriginalCapital) return null
 
         if (unit.currentMovement <= 0 || !tile.canBeSettled())
-            return UnitAction(UnitActionType.FoundCity, action = null)
+            return UnitAction(UnitActionType.FoundCity, 80f, action = null)
 
         val hasActionModifiers = unique.conditionals.any { it.type?.targetTypes?.contains(
             UniqueTarget.UnitActionModifier
@@ -56,7 +55,7 @@ object UnitActionsFromUniques {
         val foundAction = {
             if (unit.civ.playerType != PlayerType.AI)
                 UncivGame.Current.settings.addCompletedTutorialTask("Found city")
-            unit.civ.addCity(tile.position)
+            unit.civ.addCity(tile.position, unit)
 
             if (hasActionModifiers) UnitActionModifiers.activateSideEffects(unit, unique)
             else unit.destroy()
@@ -64,7 +63,7 @@ object UnitActionsFromUniques {
         }
 
         if (unit.civ.playerType == PlayerType.AI)
-            return UnitAction(UnitActionType.FoundCity, action = foundAction)
+            return UnitAction(UnitActionType.FoundCity, 80f, action = foundAction)
 
         val title =
             if (hasActionModifiers) UnitActionModifiers.actionTextWithSideEffects(
@@ -76,6 +75,7 @@ object UnitActionsFromUniques {
 
         return UnitAction(
             type = UnitActionType.FoundCity,
+            useFrequency = 80f,
             title = title,
             uncivSound = UncivSound.Chimes,
             action = {
@@ -93,7 +93,7 @@ object UnitActionsFromUniques {
                         action = foundAction
                     ).open(force = true)
                 }
-            }
+            }.takeIf { UnitActionModifiers.canActivateSideEffects(unit, unique) }
         )
     }
 
@@ -117,11 +117,12 @@ object UnitActionsFromUniques {
         return if(leadersWePromisedNotToSettleNear.isEmpty()) null else leadersWePromisedNotToSettleNear.joinToString(", ")
     }
 
-    fun getSetupActions(unit: MapUnit, tile: Tile): List<UnitAction> {
-        if (!unit.hasUnique(UniqueType.MustSetUp) || unit.isEmbarked()) return emptyList()
+    internal fun getSetupActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
+        if (!unit.hasUnique(UniqueType.MustSetUp) || unit.isEmbarked()) return emptySequence()
         val isSetUp = unit.isSetUpForSiege()
-        return listOf(UnitAction(UnitActionType.SetUp,
+        return sequenceOf(UnitAction(UnitActionType.SetUp,
             isCurrentAction = isSetUp,
+            useFrequency = 85f,
             action = {
                 unit.action = UnitActionType.SetUp.value
                 unit.useMovementPoints(1f)
@@ -129,13 +130,14 @@ object UnitActionsFromUniques {
         )
     }
 
-    fun getParadropActions(unit: MapUnit, tile: Tile): List<UnitAction> {
+    internal fun getParadropActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
         val paradropUniques =
             unit.getMatchingUniques(UniqueType.MayParadrop)
-        if (!paradropUniques.any() || unit.isEmbarked()) return emptyList()
+        if (!paradropUniques.any() || unit.isEmbarked()) return emptySequence()
         unit.cache.paradropRange = paradropUniques.maxOfOrNull { it.params[0] }!!.toInt()
-        return listOf(UnitAction(UnitActionType.Paradrop,
+        return sequenceOf(UnitAction(UnitActionType.Paradrop,
             isCurrentAction = unit.isPreparingParadrop(),
+            useFrequency = 60f, // While it is important to see, it isn't nessesary used a lot
             action = {
                 if (unit.isPreparingParadrop()) unit.action = null
                 else unit.action = UnitActionType.Paradrop.value
@@ -147,12 +149,13 @@ object UnitActionsFromUniques {
         )
     }
 
-    fun getAirSweepActions(unit: MapUnit, tile: Tile): List<UnitAction> {
+    internal fun getAirSweepActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
         val airsweepUniques =
             unit.getMatchingUniques(UniqueType.CanAirsweep)
-        if (!airsweepUniques.any()) return emptyList()
-        return listOf(UnitAction(UnitActionType.AirSweep,
+        if (!airsweepUniques.any()) return emptySequence()
+        return sequenceOf(UnitAction(UnitActionType.AirSweep,
             isCurrentAction = unit.isPreparingAirSweep(),
+            useFrequency = 90f,
             action = {
                 if (unit.isPreparingAirSweep()) unit.action = null
                 else unit.action = UnitActionType.AirSweep.value
@@ -161,7 +164,8 @@ object UnitActionsFromUniques {
             }
         ))
     }
-    fun getTriggerUniqueActions(unit: MapUnit, tile: Tile) = sequence {
+
+    internal fun getTriggerUniqueActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getUniques()) {
             // not a unit action
             if (unique.conditionals.none { it.type?.targetTypes?.contains(UniqueTarget.UnitActionModifier) == true }) continue
@@ -171,26 +175,60 @@ object UnitActionsFromUniques {
             if (!unique.conditionalsApply(StateForConditionals(civInfo = unit.civ, unit = unit, tile = unit.currentTile))) continue
             if (!UnitActionModifiers.canUse(unit, unique)) continue
 
-            val baseTitle = if (unique.isOfType(UniqueType.OneTimeEnterGoldenAgeTurns))
-                unique.placeholderText.fillPlaceholders(
-                    unit.civ.goldenAges.calculateGoldenAgeLength(
-                        unique.params[0].toInt()).toString())
-            else unique.text.removeConditionals()
+            val baseTitle = when (unique.type) {
+                UniqueType.OneTimeEnterGoldenAgeTurns -> {
+                    unique.placeholderText.fillPlaceholders(
+                        unit.civ.goldenAges.calculateGoldenAgeLength(
+                            unique.params[0].toInt()).toString())
+                    }
+                UniqueType.OneTimeGainStatSpeed -> {
+                    val stat = unique.params[1]
+                    val modifier = unit.civ.gameInfo.speed.statCostModifiers[Stat.safeValueOf(stat)]
+                        ?: unit.civ.gameInfo.speed.modifier
+                    UniqueType.OneTimeGainStat.placeholderText.fillPlaceholders(
+                        (unique.params[0].toInt() * modifier).toInt().toString(), stat
+                    )
+                }
+                UniqueType.OneTimeGainStatRange -> {
+                    val stat = unique.params[2]
+                    val modifier = unit.civ.gameInfo.speed.statCostModifiers[Stat.safeValueOf(stat)]
+                        ?: unit.civ.gameInfo.speed.modifier
+                    unique.placeholderText.fillPlaceholders(
+                        (unique.params[0].toInt() * modifier).toInt().toString(),
+                        (unique.params[1].toInt() * modifier).toInt().toString(),
+                        stat
+                    )
+                }
+                else -> unique.text.removeConditionals()
+            }
             val title = UnitActionModifiers.actionTextWithSideEffects(baseTitle, unique, unit)
 
-            yield(UnitAction(UnitActionType.TriggerUnique, title) {
-                UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
-                UnitActionModifiers.activateSideEffects(unit, unique)
-            })
-        }
-    }.asIterable()
+            val unitAction = fun (): (()->Unit)? {
+                if (unit.currentMovement == 0f) return null
+                val triggerFunction = UniqueTriggerActivation.getTriggerFunction(unique, unit.civ, unit = unit, tile = unit.currentTile)
+                    ?: return null
+                return { // This is the *action* that will be triggered!
+                    triggerFunction.invoke()
+                    UnitActionModifiers.activateSideEffects(unit, unique)
+                }
+            }()
 
-    fun getAddInCapitalActions(unit: MapUnit, tile: Tile): List<UnitAction> {
-        if (!unit.hasUnique(UniqueType.AddInCapital)) return listOf()
-        return listOf(UnitAction(UnitActionType.AddInCapital,
+            yield(
+                UnitAction(UnitActionType.TriggerUnique, 80f, title,
+                    action = unitAction.takeIf {
+                        UnitActionModifiers.canActivateSideEffects(unit, unique)
+                    })
+            )
+        }
+    }
+
+    internal fun getAddInCapitalActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
+        if (!unit.hasUnique(UniqueType.AddInCapital)) return emptySequence()
+        return sequenceOf(UnitAction(UnitActionType.AddInCapital,
             title = "Add to [${
                 unit.getMatchingUniques(UniqueType.AddInCapital).first().params[0]
             }]",
+            useFrequency = 80f,
             action = {
                 unit.civ.victoryManager.currentsSpaceshipParts.add(unit.name, 1)
                 unit.destroy()
@@ -201,28 +239,28 @@ object UnitActionsFromUniques {
         ))
     }
 
-    fun getImprovementCreationActions(unit: MapUnit, tile: Tile) = sequence {
+    internal fun getImprovementCreationActions(unit: MapUnit, tile: Tile) = sequence {
         val waterImprovementAction = getWaterImprovementAction(unit, tile)
         if (waterImprovementAction != null) yield(waterImprovementAction)
         yieldAll(getImprovementConstructionActionsFromGeneralUnique(unit, tile))
-    }.asIterable()
+    }
 
-    fun getWaterImprovementAction(unit: MapUnit, tile: Tile): UnitAction? {
+    private fun getWaterImprovementAction(unit: MapUnit, tile: Tile): UnitAction? {
         if (!tile.isWater || !unit.hasUnique(UniqueType.CreateWaterImprovements) || tile.resource == null) return null
 
         val improvementName = tile.tileResource.getImprovingImprovement(tile, unit.civ) ?: return null
         val improvement = tile.ruleset.tileImprovements[improvementName] ?: return null
         if (!tile.improvementFunctions.canBuildImprovement(improvement, unit.civ)) return null
 
-        return UnitAction(UnitActionType.CreateImprovement, "Create [$improvementName]",
+        return UnitAction(UnitActionType.CreateImprovement, 82f, "Create [$improvementName]",
             action = {
                 tile.changeImprovement(improvementName, unit.civ, unit)
                 unit.destroy()  // Modders may wish for a nondestructive way, but that should be another Unique
             }.takeIf { unit.currentMovement > 0 })
     }
 
-    fun getImprovementConstructionActionsFromGeneralUnique(unit: MapUnit, tile: Tile): ArrayList<UnitAction> {
-        val finalActions = ArrayList<UnitAction>()
+    // Not internal: Used in SpecificUnitAutomation
+    fun getImprovementConstructionActionsFromGeneralUnique(unit: MapUnit, tile: Tile) = sequence {
         val uniquesToCheck = UnitActionModifiers.getUsableUnitActionUniques(unit, UniqueType.ConstructImprovementInstantly)
 
         val civResources = unit.civ.getCivResourcesByName()
@@ -237,12 +275,11 @@ object UnitActionsFromUniques {
                 if (tile.improvementFunctions.getImprovementBuildingProblems(improvement, unit.civ).any { it.permanent })
                     continue
 
-                val resourcesAvailable = improvement.uniqueObjects.none { improvementUnique ->
-                    improvementUnique.isOfType(UniqueType.ConsumesResources) &&
+                val resourcesAvailable = improvement.getMatchingUniques(UniqueType.ConsumesResources).none { improvementUnique ->
                         (civResources[improvementUnique.params[1]] ?: 0) < improvementUnique.params[0].toInt()
                 }
 
-                finalActions += UnitAction(UnitActionType.CreateImprovement,
+                yield(UnitAction(UnitActionType.CreateImprovement, 85f,
                     title = UnitActionModifiers.actionTextWithSideEffects(
                         "Create [${improvement.name}]",
                         unique,
@@ -262,14 +299,14 @@ object UnitActionsFromUniques {
                             // Next test is to prevent interfering with UniqueType.CreatesOneImprovement -
                             // not pretty, but users *can* remove the building from the city queue an thus clear this:
                             && !tile.isMarkedForCreatesOneImprovement()
-                            && !tile.isImpassible() // Not 100% sure that this check is necessary...
-                    })
+                            && UnitActionModifiers.canActivateSideEffects(unit, unique)
+                    }
+                ))
             }
         }
-        return finalActions
     }
 
-    fun getConnectRoadActions(unit: MapUnit, tile: Tile) = sequence {
+    internal fun getConnectRoadActions(unit: MapUnit, tile: Tile) = sequence {
         if (!unit.hasUnique(UniqueType.BuildImprovements)) return@sequence
         val unitCivBestRoad = unit.civ.tech.getBestRoadAvailable()
         if (unitCivBestRoad == RoadStatus.None) return@sequence
@@ -278,14 +315,14 @@ object UnitActionsFromUniques {
 
         // If a unit has terrainFilter "Land" or improvementFilter "All", then we may proceed.
         // If a unit only had improvement filter "Road" or "Railroad", then we need to also check if that tech is unlocked
-        val unitCanBuildRoad = uniquesToCheck.any { it.params[0] == "Land" || it.params[0] == "All" }
+        val unitCanBuildRoad = uniquesToCheck.any { it.params[0] == "Land" || it.params[0] in Constants.all }
             || uniquesToCheck.any {it.params[0] == "Road" } && (unitCivBestRoad == RoadStatus.Road || unitCivBestRoad == RoadStatus.Railroad)
             || uniquesToCheck.any {it.params[0] == "Railroad"} && (unitCivBestRoad == RoadStatus.Railroad)
 
         if(!unitCanBuildRoad) return@sequence
 
         val worldScreen = GUI.getWorldScreen()
-        yield(UnitAction(UnitActionType.ConnectRoad,
+        yield(UnitAction(UnitActionType.ConnectRoad, 25f, // Press once for a multiturn command, it doesn't need to be used that frequently
                isCurrentAction = unit.isAutomatingRoadConnection(),
                action = {
                    worldScreen.bottomUnitTable.selectedUnitIsConnectingRoad =
@@ -294,25 +331,22 @@ object UnitActionsFromUniques {
                }
            )
         )
-    }.asIterable()
+    }
 
-    fun getTransformActions(
-        unit: MapUnit, tile: Tile
-    ): ArrayList<UnitAction> {
+    internal fun getTransformActions(unit: MapUnit, tile: Tile) = sequence {
         val unitTile = unit.getTile()
         val civInfo = unit.civ
         val stateForConditionals =
             StateForConditionals(unit = unit, civInfo = civInfo, tile = unitTile)
-        val transformList = ArrayList<UnitAction>()
+
         for (unique in unit.getMatchingUniques(UniqueType.CanTransform, stateForConditionals)) {
             val unitToTransformTo = civInfo.getEquivalentUnit(unique.params[0])
 
+            // Respect OnlyAvailable criteria
             if (unitToTransformTo.getMatchingUniques(
-                    UniqueType.OnlyAvailableWhen,
-                    StateForConditionals.IgnoreConditionals
-                )
-                    .any { !it.conditionalsApply(stateForConditionals) })
-                continue
+                    UniqueType.OnlyAvailable, StateForConditionals.IgnoreConditionals
+                ).any { !it.conditionalsApply(stateForConditionals) }
+            ) continue
 
             // Check _new_ resource requirements
             // Using Counter to aggregate is a bit exaggerated, but - respect the mad modder.
@@ -325,21 +359,19 @@ object UnitActionsFromUniques {
                 .filter { it.value > 0 }
                 .joinToString { "${it.value} {${it.key}}".tr() }
 
-            val title = if (newResourceRequirementsString.isEmpty())
-                "Transform to [${unitToTransformTo.name}]"
-            else "Transform to [${unitToTransformTo.name}]\n([$newResourceRequirementsString])"
+            var title = "Transform to [${unitToTransformTo.name}] "
+            title += UnitActionModifiers.getSideEffectString(unit, unique, true)
+            if (newResourceRequirementsString.isNotEmpty())
+                title += "\n([$newResourceRequirementsString])"
 
-            transformList.add(UnitAction(UnitActionType.Transform,
+            yield(UnitAction(UnitActionType.Transform, 70f,
                 title = title,
                 action = {
+                    val oldMovement = unit.currentMovement
                     unit.destroy()
                     val newUnit =
                         civInfo.units.placeUnitNearTile(unitTile.position, unitToTransformTo)
 
-                    /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
-                     * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
-                     */
-
                     /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
                      * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
                      */
@@ -349,21 +381,25 @@ object UnitActionsFromUniques {
                         unit.copyStatisticsTo(resurrectedUnit)
                     } else { // Managed to upgrade
                         unit.copyStatisticsTo(newUnit)
-                        newUnit.currentMovement = 0f
+                        // have to handle movement manually because we killed the old unit
+                        // a .destroy() unit has 0 movement
+                        // and a new one may have less Max Movement
+                        newUnit.currentMovement = oldMovement
+                        // adjust if newUnit has lower Max Movement
+                        if (newUnit.currentMovement.toInt() > newUnit.getMaxMovement())
+                            newUnit.currentMovement = newUnit.getMaxMovement().toFloat()
+                        // execute any side effects, Stat and Movement adjustments
+                        UnitActionModifiers.activateSideEffects(newUnit, unique, true)
                     }
                 }.takeIf {
-                    unit.currentMovement > 0 && !unit.isEmbarked()
+                    !unit.isEmbarked() && UnitActionModifiers.canActivateSideEffects(unit, unique)
                 }
             ))
         }
-        return transformList
     }
 
-    fun getBuildingImprovementsActions(
-        unit: MapUnit,
-        tile: Tile
-    ): List<UnitAction> {
-        if (!unit.cache.hasUniqueToBuildImprovements) return emptyList()
+    internal fun getBuildingImprovementsActions(unit: MapUnit, tile: Tile): Sequence<UnitAction> {
+        if (!unit.cache.hasUniqueToBuildImprovements) return emptySequence()
 
         val couldConstruct = unit.currentMovement > 0
             && !tile.isCityCenter()
@@ -377,7 +413,7 @@ object UnitActionsFromUniques {
                 && unit.canBuildImprovement(it)
         }
 
-        return listOf(UnitAction(UnitActionType.ConstructImprovement,
+        return sequenceOf(UnitAction(UnitActionType.ConstructImprovement, 85f,
             isCurrentAction = tile.hasImprovementInProgress(),
             action = {
                 GUI.pushScreen(ImprovementPickerScreen(tile, unit) {
@@ -388,7 +424,7 @@ object UnitActionsFromUniques {
         ))
     }
 
-    fun getRepairTurns(unit: MapUnit): Int {
+    internal fun getRepairTurns(unit: MapUnit): Int {
         val tile = unit.currentTile
         if (!tile.isPillaged()) return 0
         if (tile.improvementInProgress == Constants.repair) return tile.turnsToImprovement
@@ -401,10 +437,9 @@ object UnitActionsFromUniques {
         return repairTurns
     }
 
-    fun getRepairActions(unit: MapUnit, tile: Tile): List<UnitAction> {
-        val repairAction = getRepairAction(unit) ?: return emptyList()
-        return listOf(repairAction)
-    }
+    internal fun getRepairActions(unit: MapUnit, tile: Tile) = sequenceOf(getRepairAction(unit)).filterNotNull()
+
+    // Public - used in WorkerAutomation
     fun getRepairAction(unit: MapUnit) : UnitAction? {
         if (!unit.currentTile.ruleset.tileImprovements.containsKey(Constants.repair)) return null
         if (!unit.cache.hasUniqueToBuildImprovements) return null
@@ -415,10 +450,11 @@ object UnitActionsFromUniques {
 
         val couldConstruct = unit.currentMovement > 0
             && !tile.isCityCenter() && tile.improvementInProgress != Constants.repair
+            && !tile.isEnemyTerritory(unit.civ)
 
         val turnsToBuild = getRepairTurns(unit)
 
-        return UnitAction(UnitActionType.Repair,
+        return UnitAction(UnitActionType.Repair, 90f,
             title = "${UnitActionType.Repair} [${unit.currentTile.getImprovementToRepair()!!.name}] - [${turnsToBuild}${Fonts.turn}]",
             action = {
                 tile.turnsToImprovement = getRepairTurns(unit)
