@@ -11,12 +11,13 @@ import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.extensions.toPercent
 import kotlin.math.min
 
+@Suppress("UNUSED_PARAMETER") // references need to have the signature expected by UnitActions.actionTypeToFunctions
 object UnitActionsGreatPerson {
 
-    fun getHurryResearchActions(unit:MapUnit, tile: Tile) = sequence {
+    internal fun getHurryResearchActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getMatchingUniques(UniqueType.CanHurryResearch)){
             yield(UnitAction(
-                UnitActionType.HurryResearch,
+                UnitActionType.HurryResearch, 76f,
                 action = {
                     unit.civ.tech.addScience(unit.civ.tech.getScienceFromGreatScientist())
                     unit.consume()
@@ -27,21 +28,21 @@ object UnitActionsGreatPerson {
                 }
             ))
         }
-    }.asIterable()
+    }
 
-    fun getHurryPolicyActions(unit:MapUnit, tile: Tile) = sequence {
+    internal fun getHurryPolicyActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getMatchingUniques(UniqueType.CanHurryPolicy)){
             yield(UnitAction(
-                UnitActionType.HurryPolicy,
+                UnitActionType.HurryPolicy, 76f,
                 action = {
                     unit.civ.policies.addCulture(unit.civ.policies.getCultureFromGreatWriter())
                     unit.consume()
                 }.takeIf {unit.currentMovement > 0}
             ))
         }
-    }.asIterable()
+    }
 
-    fun getHurryWonderActions(unit: MapUnit, tile: Tile) = sequence {
+    internal fun getHurryWonderActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getMatchingUniques(UniqueType.CanSpeedupWonderConstruction)) {
             val canHurryWonder =
                 if (!tile.isCityCenter()) false
@@ -49,7 +50,7 @@ object UnitActionsGreatPerson {
                     && tile.getCity()!!.cityConstructions.canBeHurried()
 
             yield(UnitAction(
-                UnitActionType.HurryWonder,
+                UnitActionType.HurryWonder, 75f,
                 action = {
                     tile.getCity()!!.cityConstructions.apply {
                         //http://civilization.wikia.com/wiki/Great_engineer_(Civ5)
@@ -61,12 +62,12 @@ object UnitActionsGreatPerson {
                 }.takeIf { unit.currentMovement > 0 && canHurryWonder }
             ))
         }
-    }.asIterable()
+    }
 
-    fun getHurryBuildingActions(unit:MapUnit, tile: Tile) = sequence {
+    internal fun getHurryBuildingActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getMatchingUniques(UniqueType.CanSpeedupConstruction)) {
             if (!tile.isCityCenter()) {
-                yield(UnitAction(UnitActionType.HurryBuilding, action = null))
+                yield(UnitAction(UnitActionType.HurryBuilding, 75f, action = null))
                 continue
             }
 
@@ -82,7 +83,7 @@ object UnitActionsGreatPerson {
             if (productionPointsToAdd <= 0) continue
 
             yield(UnitAction(
-                UnitActionType.HurryBuilding,
+                UnitActionType.HurryBuilding, 75f,
                 title = "Hurry Construction (+[$productionPointsToAdd]⚙)",
                 action = {
                     cityConstructions.apply {
@@ -94,9 +95,9 @@ object UnitActionsGreatPerson {
                 }.takeIf { unit.currentMovement > 0 && canHurryConstruction }
             ))
         }
-    }.asIterable()
+    }
 
-    fun getConductTradeMissionActions(unit:MapUnit, tile: Tile) = sequence {
+    internal fun getConductTradeMissionActions(unit: MapUnit, tile: Tile) = sequence {
         for (unique in unit.getMatchingUniques(UniqueType.CanTradeWithCityStateForGoldAndInfluence)) {
             val canConductTradeMission = tile.owningCity?.civ?.isCityState() == true
                 && tile.owningCity?.civ != unit.civ
@@ -104,7 +105,7 @@ object UnitActionsGreatPerson {
             val influenceEarned = unique.params[0].toFloat()
 
             yield(UnitAction(
-                UnitActionType.ConductTradeMission,
+                UnitActionType.ConductTradeMission, 70f,
                 action = {
                     // http://civilization.wikia.com/wiki/Great_Merchant_(Civ5)
                     var goldEarned = (350 + 50 * unit.civ.getEraNumber()) * unit.civ.gameInfo.speed.goldCostModifier
@@ -120,5 +121,5 @@ object UnitActionsGreatPerson {
                 }.takeIf { unit.currentMovement > 0 && canConductTradeMission }
             ))
         }
-    }.asIterable()
+    }
 }
